@@ -1,6 +1,8 @@
 // -*- mode:C++; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil -*-
-/* Blackjack - events.cpp
- * Copyright (C) 2003 William Jon McCann <mccann@jhu.edu>
+/*
+ * Blackjack - events.cpp
+ *
+ * Copyright (C) 2003-2004 William Jon McCann <mccann@jhu.edu>
  *
  * This game is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +34,10 @@
 #include <ctype.h>
 #include <iostream>
 #include <math.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
+
 using namespace std;
 
 #include "events.h"
@@ -42,10 +48,49 @@ using namespace std;
 #include "card.h"
 #include "menu.h"
 #include "chips.h"
+#include "press_data.h"
 
 #include "hand.h"
 #include "game.h"
 
+/* These have been lifted straight from eog. */
+  	 
+#define hand_closed_data_width 20
+#define hand_closed_data_height 20
+static char hand_closed_data_bits[] = {
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x00, 0x80, 0x3f, 0x00,
+        0x80, 0xff, 0x00, 0x80, 0xff, 0x00, 0xb0, 0xff, 0x00, 0xf0, 0xff, 0x00,
+        0xe0, 0xff, 0x00, 0xe0, 0x7f, 0x00, 0xc0, 0x7f, 0x00, 0x80, 0x3f, 0x00,
+        0x00, 0x3f, 0x00, 0x00, 0x3f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+  	 
+#define hand_closed_mask_width 20
+#define hand_closed_mask_height 20
+static char hand_closed_mask_bits[] = {
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+        0x00, 0x00, 0x00, 0x00, 0x06, 0x00, 0x80, 0x3f, 0x00, 0xc0, 0xff, 0x00,
+        0xc0, 0xff, 0x01, 0xf0, 0xff, 0x01, 0xf8, 0xff, 0x01, 0xf8, 0xff, 0x01,
+        0xf0, 0xff, 0x01, 0xf0, 0xff, 0x00, 0xe0, 0xff, 0x00, 0xc0, 0x7f, 0x00,
+        0x80, 0x7f, 0x00, 0x80, 0x7f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+  	 
+#define hand_open_data_width 20
+#define hand_open_data_height 20
+static char hand_open_data_bits[] = {
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x00,
+        0x60, 0x36, 0x00, 0x60, 0x36, 0x00, 0xc0, 0x36, 0x01, 0xc0, 0xb6, 0x01,
+        0x80, 0xbf, 0x01, 0x98, 0xff, 0x01, 0xb8, 0xff, 0x00, 0xf0, 0xff, 0x00,
+        0xe0, 0xff, 0x00, 0xe0, 0x7f, 0x00, 0xc0, 0x7f, 0x00, 0x80, 0x3f, 0x00,
+        0x00, 0x3f, 0x00, 0x00, 0x3f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+  	 
+#define hand_open_mask_width 20
+#define hand_open_mask_height 20
+static char hand_open_mask_bits[] = {
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x00, 0x60, 0x3f, 0x00,
+        0xf0, 0x7f, 0x00, 0xf0, 0x7f, 0x01, 0xe0, 0xff, 0x03, 0xe0, 0xff, 0x03,
+        0xd8, 0xff, 0x03, 0xfc, 0xff, 0x03, 0xfc, 0xff, 0x01, 0xf8, 0xff, 0x01,
+        0xf0, 0xff, 0x01, 0xf0, 0xff, 0x00, 0xe0, 0xff, 0x00, 0xc0, 0x7f, 0x00,
+        0x80, 0x7f, 0x00, 0x80, 0x7f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+
 hslot_type last_hslot = NULL;
 gint last_cardid = -1;
 
@@ -55,6 +100,45 @@ gint last_chipid = -1;
 GTimer *click_timer = NULL;
 gdouble dbl_click_time;
 
+GdkCursor * hand_closed_cursor = NULL;
+GdkCursor * hand_open_cursor = NULL;
+GdkCursor * default_cursor = NULL;
+  	 
+enum {
+        CURSOR_NONE = 0,
+        CURSOR_OPEN,
+        CURSOR_CLOSED
+};
+  	 
+static void
+set_cursor (int cursor)
+{
+        if (cursor == CURSOR_NONE)
+                gdk_window_set_cursor (playing_area->window, default_cursor);
+        if (cursor == CURSOR_OPEN)
+                gdk_window_set_cursor (playing_area->window, hand_open_cursor);
+        if (cursor == CURSOR_CLOSED)
+                gdk_window_set_cursor (playing_area->window, hand_closed_cursor);
+}
+  	 
+/* If we are over a slot, set the cursor to the given cursor,
+ * otherwise use the default cursor. */
+static void
+set_cursor_by_location (int cursor, int x, int y)
+{
+        hslot_type hslot;
+        gint cardid;
+        hstack_type hstack;
+        gint chipid; 
+
+        bj_slot_pressed (x, y, &hslot, &cardid);
+        bj_chip_stack_pressed (x, y, &hstack, &chipid);
+        if (hslot || hstack)
+                set_cursor (cursor);
+        else
+                set_cursor (CURSOR_NONE);
+}
+
 gboolean
 bj_event_key_press (GtkWidget *widget, GdkEventKey *event, void *d)
 {
@@ -107,10 +191,6 @@ waiting_for_mouse_up (void)
 
 /* Button press statuses */
 
-void end_of_game_test ()
-{
-}
-
 static void
 drop_moving_cards (gint x, gint y)
 {
@@ -143,16 +223,9 @@ drop_moving_cards (gint x, gint y)
                          hslot->pixelx + hslot->width - width, 
                          hslot->pixely + hslot->height - height);
 
-        bj_draw_refresh_screen ();
+        bj_press_data_free ();
 
-        gdk_window_hide (press_data->moving_cards);
-  
-        if (press_data->moving_pixmap)
-                g_object_unref (press_data->moving_pixmap);
-        if (press_data->moving_mask)
-                g_object_unref (press_data->moving_mask);
-        press_data->moving_pixmap = NULL;
-        press_data->moving_mask = NULL;
+        bj_draw_refresh_screen ();
 
         if (do_split)
                 bj_hand_split ();
@@ -202,18 +275,8 @@ drop_moving_chips (gint x, gint y)
                 bj_chip_stack_update_length (chip_stack_press_data->hstack);
         }
 
-        chip_stack_press_data->chips = NULL;
-
+        bj_chip_stack_press_data_free ();
         bj_draw_refresh_screen ();
-
-        gdk_window_hide (chip_stack_press_data->moving_chips);
-  
-        if (chip_stack_press_data->moving_pixmap)
-                g_object_unref (chip_stack_press_data->moving_pixmap);
-        if (chip_stack_press_data->moving_mask)
-                g_object_unref (chip_stack_press_data->moving_mask);
-        chip_stack_press_data->moving_pixmap = NULL;
-        chip_stack_press_data->moving_mask = NULL;
 }
 
 static gint
@@ -264,7 +327,6 @@ handle_slot_pressed (GdkEventButton *event, hslot_type hslot, gint cardid)
         else if (double_click) {
                 press_data->status = STATUS_NONE;
                 //bj_draw_refresh_screen ();
-                //end_of_game_test ();
                 return TRUE;
         }
         return TRUE;
@@ -308,7 +370,7 @@ handle_chip_stack_pressed (GdkEventButton *event,
                 int x = hstack->pixelx + delta * hstack->pixeldx;
                 int y = hstack->pixely + delta * hstack->pixeldy;
                 
-                press_data->status = STATUS_SHOW;
+                chip_stack_press_data->status = STATUS_SHOW;
                 gfloat chip_value = 
                         ((hchip_type)g_list_nth_data (hstack->chips, chipid))->value;
                 
@@ -316,16 +378,16 @@ handle_chip_stack_pressed (GdkEventButton *event,
                 GdkPixmap *pixmap;
                 GdkBitmap *mask;
                 gdk_pixbuf_render_pixmap_and_mask (pixbuf, &pixmap, &mask, 127);
-                press_data->moving_pixmap = pixmap;
-                press_data->moving_mask = mask;
+                chip_stack_press_data->moving_pixmap = pixmap;
+                chip_stack_press_data->moving_mask = mask;
                 
-                if (press_data->moving_pixmap != NULL)
-                        gdk_window_set_back_pixmap (press_data->moving_cards, 
-                                                    press_data->moving_pixmap, 0);
-                gdk_window_shape_combine_mask (press_data->moving_cards, 
-                                               press_data->moving_mask, 0, 0);
-                gdk_window_move (press_data->moving_cards, x, y); 
-                gdk_window_show (press_data->moving_cards);
+                if (chip_stack_press_data->moving_pixmap != NULL)
+                        gdk_window_set_back_pixmap (chip_stack_press_data->moving_chips, 
+                                                    chip_stack_press_data->moving_pixmap, 0);
+                gdk_window_shape_combine_mask (chip_stack_press_data->moving_chips, 
+                                               chip_stack_press_data->moving_mask, 0, 0);
+                gdk_window_move (chip_stack_press_data->moving_chips, x, y); 
+                gdk_window_show (chip_stack_press_data->moving_chips);
         } 
         else if (double_click) {
                 chip_stack_press_data->status = STATUS_NONE;
@@ -405,10 +467,7 @@ bj_event_button_release (GtkWidget *widget, GdkEventButton *event, void *d)
                 drop_moving_cards ((gint)event->x, (gint)event->y);
                 break;
         case STATUS_SHOW:
-                press_data->status = STATUS_NONE;
-                gdk_window_hide (press_data->moving_cards);
-                press_data->moving_pixmap = NULL;
-                press_data->moving_mask = NULL;
+                bj_press_data_free ();
                 break;
     
         case STATUS_MAYBE_DRAG:
@@ -428,10 +487,7 @@ bj_event_button_release (GtkWidget *widget, GdkEventButton *event, void *d)
                 drop_moving_chips ((gint)event->x, (gint)event->y);
                 break;
         case STATUS_SHOW:
-                chip_stack_press_data->status = STATUS_NONE;
-                gdk_window_hide (chip_stack_press_data->moving_chips);
-                chip_stack_press_data->moving_pixmap = NULL;
-                chip_stack_press_data->moving_mask = NULL;
+                bj_chip_stack_press_data_free ();
                 break;
     
         case STATUS_MAYBE_DRAG:
@@ -444,6 +500,9 @@ bj_event_button_release (GtkWidget *widget, GdkEventButton *event, void *d)
         case STATUS_NONE:
                 break;
         }
+
+        set_cursor_by_location (CURSOR_OPEN, (int)event->x, (int)event->y);
+
         return TRUE;
 }
 
@@ -451,33 +510,40 @@ static gint
 handle_other_motion_event (GtkWidget *widget, GdkEventMotion *event)
 {
         // This is primarily to display help information
-
         hstack_type hstack;
         gint chipid;
+        guint context_id;
+
+        context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (status_bar), "help");
+
         bj_chip_stack_pressed ((gint)(event->x), (gint)(event->y), &hstack, &chipid);
         if (hstack) {
                 if (bj_chip_stack_is_source (hstack)) {
+                        gtk_statusbar_pop (GTK_STATUSBAR (status_bar), context_id);
+
                         if (bj_hand_can_be_doubled ())
-                                gnome_appbar_set_status (GNOME_APPBAR (status_bar), 
-                                                         _("Click to double your wager"));
+                                gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id,
+                                                    _("Click to double your wager"));
                         else if (! bj_game_is_active ()) {
                                 gfloat chip_value = 
                                         ((hchip_type)g_list_last (hstack->chips)->data)->value;
                                 gchar *message = g_strdup_printf
                                         (_("Double click to increase your wager by %.2f"), 
                                          chip_value);
-                                gnome_appbar_set_status (GNOME_APPBAR (status_bar), 
-                                                         message);
+                                gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id,
+                                                    message);
                                 g_free (message);
                         }
                 }
                 else {
                         if (! bj_game_is_active ()) {
+                                gtk_statusbar_pop (GTK_STATUSBAR (status_bar), context_id);
+
                                 gfloat chip_value = ((hchip_type)g_list_last (hstack->chips)->data)->value;
                                 gchar *message = g_strdup_printf (_("Double click to decrease your wager by %.2f"), 
                                                                   chip_value);
-                                gnome_appbar_set_status (GNOME_APPBAR (status_bar), 
-                                                         message);
+                                gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id,
+                                                    message);
                                 g_free (message);
                         }
                 }
@@ -499,13 +565,14 @@ handle_other_motion_event (GtkWidget *widget, GdkEventMotion *event)
                         }
                         else
                                 message = g_strdup (_("Click to deal a new hand"));
-                        gnome_appbar_set_status (GNOME_APPBAR (status_bar), message);
+                        gtk_statusbar_pop (GTK_STATUSBAR (status_bar), context_id);
+                        gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id,
+                                            message);
                         g_free (message);
                 }
                 else {
                         // Not over anything we know about
-                        // black out the status bar
-                        gnome_appbar_set_status (GNOME_APPBAR (status_bar), "");
+                        gtk_statusbar_pop (GTK_STATUSBAR (status_bar), context_id);
                 }
         }
         return FALSE;
@@ -519,6 +586,7 @@ handle_slot_motion_event (GtkWidget *widget, GdkEventMotion *event)
                                  (gint)event->x - press_data->xoffset,
                                  (gint)event->y - press_data->yoffset);
                 gdk_window_clear (press_data->moving_cards);
+                set_cursor (CURSOR_CLOSED);
                 return TRUE;
         }
         else if (press_data->status == STATUS_MAYBE_DRAG
@@ -527,7 +595,10 @@ handle_slot_motion_event (GtkWidget *widget, GdkEventMotion *event)
                 press_data->status = STATUS_IS_DRAG;
                 bj_press_data_generate ();
                 bj_draw_take_snapshot();
+                set_cursor (CURSOR_CLOSED);
                 return TRUE;
+        } else {
+                set_cursor_by_location (CURSOR_OPEN, (int)event->x, (int)event->y);
         }
   
         return FALSE;
@@ -541,6 +612,7 @@ handle_chip_stack_motion_event (GtkWidget *widget, GdkEventMotion *event)
                                  (gint)event->x - chip_stack_press_data->xoffset,
                                  (gint)event->y - chip_stack_press_data->yoffset);
                 gdk_window_clear (chip_stack_press_data->moving_chips);
+                set_cursor (CURSOR_CLOSED);
                 return TRUE;
         }
         else if (chip_stack_press_data->status == STATUS_MAYBE_DRAG
@@ -549,9 +621,13 @@ handle_chip_stack_motion_event (GtkWidget *widget, GdkEventMotion *event)
                 chip_stack_press_data->status = STATUS_IS_DRAG;
                 bj_chip_stack_press_data_generate ();
                 bj_draw_take_snapshot();
+                set_cursor (CURSOR_CLOSED);
                 return TRUE;
+        } else {
+                set_cursor_by_location (CURSOR_OPEN, (int)event->x, (int)event->y);
         }
   
+  
         return FALSE;
 }
 
@@ -581,16 +657,49 @@ bj_event_expose_callback (GtkWidget *widget, GdkEventExpose *event, void *d)
         return TRUE;
 }
 
+static GdkCursor *
+make_cursor (char * data, char * mask_data)
+{
+        GdkColor fg = {0, 65535, 65535, 65535};
+        GdkColor bg = {0, 0, 0, 0};
+        GdkPixmap * source;
+        GdkPixmap * mask;
+        GdkCursor * cursor;
+  	 
+        /* Yeah, hard-coded sizes are bad. */
+        source = gdk_bitmap_create_from_data (NULL, data, 20, 20);
+        mask = gdk_bitmap_create_from_data (NULL, mask_data, 20, 20);
+  	 
+        cursor = gdk_cursor_new_from_pixmap (source, mask, &fg, &bg, 10, 10);
+  	 
+        g_object_unref (source);
+        g_object_unref (mask);
+  	 
+        return cursor;
+}
+
 gint
 bj_event_playing_area_configure (GtkWidget *widget, GdkEventConfigure *event)
 {
         gint tmptime;
         GtkSettings *settings;
 
+        if (!default_cursor)
+                default_cursor = gdk_cursor_new(GDK_LEFT_PTR);
+  	 
+        if (!hand_closed_cursor)
+                hand_closed_cursor = make_cursor (hand_closed_data_bits,
+                                                  hand_closed_mask_bits);
+  	 
+        if (!hand_open_cursor)
+                hand_open_cursor = make_cursor (hand_open_data_bits,
+                                                hand_open_mask_bits);
+
         if (surface) {
                 if (window_width == event->width && window_height == event->height)
                         return TRUE;
                 g_object_unref (surface);
+                surface = NULL;
         }
   
         if (!draw_gc) {
