@@ -357,6 +357,12 @@ Tetris::setupPixmap()
 	fullpixname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, pixname, FALSE, NULL);
 	g_free (pixname);
 
+	if (!g_file_test (fullpixname, G_FILE_TEST_EXISTS))
+	{
+		g_free (fullpixname);
+		fullpixname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, "gnometris/7blocks-gw.png", FALSE, NULL);
+	}
+	
 	if (!g_file_test (fullpixname, G_FILE_TEST_EXISTS))
 	{
 		GtkWidget *widget = gtk_message_dialog_new (NULL,
